#ifndef _KVI_IRCSERVERDB_H_
#define _KVI_IRCSERVERDB_H_
//=============================================================================
//
//   File : kvi_ircserverdb.h
//   Creation date : Mon Jul 10 2000 14:15:42 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2004 Szymon Stefanek (pragma at kvirc dot net)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "kvi_settings.h"
#include "kvi_qstring.h"
#include "kvi_ircserver.h"

#include <qdialog.h>

typedef struct _KviIrcServerDefinition
{
	QString            szServer;
	unsigned short int uPort;
	bool               bPortIsValid;
	bool               bIpV6;
	bool               bSSL;
	QString            szLinkFilter;
	QString            szPass;
	QString            szNick;
	QString            szInitUMode;
} KviIrcServerDefinition;




class KVILIB_API KviIrcServerDataBase
{
public:
	KviIrcServerDataBase();
	~KviIrcServerDataBase();
private:
	KviPtrList<KviIrcNetwork> * m_pNetworkList;
	KviIrcNetwork             * m_pCurrentNetwork;
	// This list is computed when the data are loaded from disk
	// during the startup and is used by KviApp to
	// start the connections.
	// The pointer is zero if there are no autoConnect servers
	// The list is valid only during the startup phase
	// because it contains shallow pointers to the servers
	// really contained in the server/network list
	// and it is never updated later
	KviPtrList<KviIrcServer>  * m_pAutoConnectOnStartupServers;
public:
	void clear();
	KviPtrList<KviIrcServer> * autoConnectOnStartupServers(){ return m_pAutoConnectOnStartupServers; };
	void clearAutoConnectOnStartupServers();
	KviPtrList<KviIrcNetwork> * networkList(){ return m_pNetworkList; };
	KviIrcNetwork * currentNetwork(){ return m_pCurrentNetwork; };
	KviIrcNetwork * findNetwork(const QString &name);
	void setCurrentNetwork(KviIrcNetwork *n);
	void load(const QString & filename);
	void save(const QString & filename);
	void insertNetwork(KviIrcNetwork * n);
	void updateServerIp(const QString & server,unsigned short int port,const QString &ip);
	void makeCurrentServer(KviIrcServerDefinition * d);
};

#endif //_KVI_IRCSERVERDB_H_
