#ifndef _KVI_IRCTOOLBAR_H_
#define _KVI_IRCTOOLBAR_H_

//
//   File : kvi_irctoolbar.h
//   Creation date : Thu Oct 29 2000 14:09:11 CEST by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2000-2001 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "kvi_settings.h"

#include "kvi_string.h"
#include "kvi_toolbar.h"
#include "kvi_console.h"

#include "kvi_list.h"
#include <qwidget.h>
#include <qtoolbutton.h>
#include <qwidgetstack.h>
#include <qlabel.h>

class QPopupMenu;


//
// Basic graphical applet
//
// Thinks about the "common" background (handles transparency stuff & co.)
// Can be a child of any AppletContainer
//

class KVIRC_API KviToolBarGraphicalApplet : public QWidget
{
	Q_OBJECT
public:
	KviToolBarGraphicalApplet(QWidget * par);
	~KviToolBarGraphicalApplet();
private:
	bool m_bResizeMode;
	bool m_bSizeLoaded;
	QSize m_sizeHint;
private:
	void resizeMemBuffer();
protected:
	void setupSizeHint();

	virtual unsigned int loadAppletWidth();
	virtual void saveAppletWidth(unsigned int uWidth);
	virtual QSize sizeHint() const;
	virtual void mouseMoveEvent(QMouseEvent *e);
	virtual void mousePressEvent(QMouseEvent *e);
	virtual void mouseReleaseEvent(QMouseEvent *e);

	virtual void paintEvent(QPaintEvent *e);
	virtual void drawContents(QPainter * p);
	virtual void resizeEvent(QResizeEvent *e);
};





class KviDynamicToolTip;
class KviFrame;
class KviConsole;

class KVIRC_API KviIrcContextDisplay : public KviToolBarGraphicalApplet
{
	Q_OBJECT
public:
	KviIrcContextDisplay(QWidget * par,KviFrame * f);
	~KviIrcContextDisplay();
protected:
	KviFrame * m_pFrm;
    bool       m_bIsLagMeterConnected;
protected:
	virtual void drawContents(QPainter * p);
	//virtual QSize sizeHint() const;
//	virtual void mousePressEvent(QMouseEvent *e);
protected slots:
	void tipRequest(KviDynamicToolTip * tip,const QPoint &);
};




class KVIRC_API KviIrcContextToolBar : public KviToolBar
{
	Q_OBJECT
public:
	KviIrcContextToolBar(KviFrame * pFrm);
	~KviIrcContextToolBar();
protected:
	KviFrame                                 * m_pFrm;

	KviConsole                               * m_pLastConsole;


	KviIrcContextDisplay                     * m_pDisplay;

	QToolButton                              * m_pConnectButton;
	QToolButton                              * m_pToolsButton;
	QToolButton                              * m_pActionsButton;

	QPopupMenu                               * m_pToolsPopup;

	QPopupMenu                               * m_pActionsPopup;

	QPopupMenu                               * m_pConnectToServerPopup;
	QPopupMenu                               * m_pChangeNicknamePopup;
	QPopupMenu                               * m_pJoinChannelPopup;
	QPopupMenu                               * m_pUserModePopup;
public:
	void activeWindowChanged(); // called by KviFrame
	void connectStateChanged(KviConsole * c); // called by KviConsole
	void updateStatusDisplay(KviConsole *c); // called by KviConsole
	void updatePseudoTransparency();
protected slots:
	void connectButtonClicked();
	void toolsButtonClicked();
	void actionsButtonClicked();

	void setupToolsPopup();

	void setupActionsPopup();
	void awayClicked();
	void setupUserModePopup();
	void setupJoinChannelPopup();
	void setupConnectToServerPopup();
	void setupChangeNicknamePopup();

	void userModeSelected(int id);
	void joinChannelSelected(int id);
	void changeNicknameSelected(int id);
	void connectToServerSelected(int id);

	void executeInternalCommand(int id);

	void reloadImages();
protected:
	void setPixmapsForConsole(KviConsole * c);
};


#endif //_KVI_IRCTOOLBAR_H_
