#ifndef _KVI_KVS_EVENTHANDLER_H_
#define _KVI_KVS_EVENTHANDLER_H_
//=============================================================================
//
//   File : kvi_kvs_eventhandler.h
//   Created on Mon 23 Feb 2004 03:04:58 by Szymon Stefanek
//
//   This file is part of the KVIrc IRC client distribution
//   Copyright (C) 2004 Szymon Stefanek <pragma at kvirc dot net>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "kvi_settings.h"
#include "kvi_kvs_script.h"
#include "kvi_qstring.h"
#include "kvi_kvs_moduleinterface.h"

class KVIRC_API KviKvsEventHandler
{
public:
	enum Type { Script , Module };
protected:
	Type m_type;
public:
	KviKvsEventHandler(Type t): m_type(t){};
	virtual ~KviKvsEventHandler(){};
public:
	Type type(){ return m_type; };
};


class KVIRC_API KviKvsScriptEventHandler : public KviKvsEventHandler
{
public:
	// the event handler becomes the owned of pszCode!
	KviKvsScriptEventHandler(const QString &szHandlerName,const QString &szContextName,QString * pszCode,bool bEnabled = true);
	virtual ~KviKvsScriptEventHandler();
protected:
	QString        m_szName;
	KviKvsScript * m_pScript;
	bool           m_bEnabled;
public:
	KviKvsScript * script(){ return m_pScript; };
	const QString & name(){ return m_szName; };
	const QString & code(){ return m_pScript->code(); };

	bool isEnabled(){ return m_bEnabled; };
	void setEnabled(bool bEnabled){ m_bEnabled = bEnabled; };
};


class KviKvsModuleEventHandler : public KviKvsEventHandler
{
protected:
	KviKvsModuleInterface           * m_pModule;
	KviKvsModuleEventHandlerRoutine * m_pProc;
public:
	KviKvsModuleEventHandler(KviKvsModuleInterface * m,KviKvsModuleEventHandlerRoutine proc);
	virtual ~KviKvsModuleEventHandler();
public:
	KviKvsModuleInterface * moduleInterface(){ return m_pModule; };
	KviKvsModuleEventHandlerRoutine * handlerRoutine(){ return m_pProc; };
};


#endif //!_KVI_KVS_EVENTHANDLER_H_
