#ifndef _KVI_SCRIPTTOOLBAR_MANAGER_H_
#define _KVI_SCRIPTTOOLBAR_MANAGER_H_
//=============================================================================
//
//   File : kvi_scripttoolbar_manager.h
//   Creation date : Fri Nov 08 2002 02:45:01 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2002 Szymon Stefanek (pragma at kvirc dot net)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "kvi_settings.h"

#ifdef COMPILE_SCRIPTTOOLBAR

	#include <qstring.h>
	#include <qdict.h>
	
	class KviApp;
	class KviMenuBar;
#ifdef COMPILE_ON_WINDOWS
	#include "kvi_scripttoolbar_definition.h"
#else
	class KviScriptToolBarDefinition;
#endif
	class KviFrame;
	
	class KVIRC_API KviScriptToolBarManager
	{
		friend class KviApp;
		friend class KviMenuBar;
		friend class KviScriptToolBarEditor;
	protected: // only KviApp can construct it
		KviScriptToolBarManager();
		virtual ~KviScriptToolBarManager();
	protected:
		QDict<KviScriptToolBarDefinition> * m_pBars;
	public:
		QDict<KviScriptToolBarDefinition> * toolBarDefinitions(){ return m_pBars; };

		void load(const char * szFileName);  // KviApp
		void save(const char * szFileName);  // KviApp
		void createVisible(KviFrame * pFrm);
		void storeVisibilityState();
	
		KviScriptToolBarDefinition * find(int id);
		KviScriptToolBarDefinition * find(const QString & szName){ return m_pBars->find(szName); };
		KviScriptToolBarDefinition * get(const QString &szName);
		KviScriptToolBarDefinition * getNew(); // finds a default valid name too
		void remove(const QString &szName);

		void clear();

		// warning : may change the name , if it collides with another definition
		void rename(KviScriptToolBarDefinition * d,const QString &szNewName);

		void copyFrom(KviScriptToolBarManager * m);
	};
	
	
	#ifndef _KVI_SCRIPTTOOLBAR_MANAGER_CPP_
		extern KVIRC_API KviScriptToolBarManager * g_pScriptToolBarManager;
	#endif

#endif // COMPILE_SCRIPTTOOLBAR

#endif // !_KVI_SCRIPTTOOLBAR_MANAGER_H_
